{  Delphi Glyph Viewer
   - View button images

    2007 J. Rathlev 24147 Klausdorf
      Web:  www.rathlev-home.de 
      Mail: info(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   J. Rathlev, D-24147 Klausdorf
   Vers. 2 - Jan 2007
   }
   
unit GlyphVMain;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, ExtCtrls, Menus, IniFiles,
  HListBox, JrUtils, XPMan;

const
  Prog = 'Glyph Viewer';
  Vers = ' - Vers. 2.0';

  IniExt = 'ini';

  BtnHeight = 26;
  BtnWidth = 26;
  BtnLeft = 5;
  BtnTop = 5;
  BtnDist = 5;

{$I ../JrGlobal.pas}
{ enthlt:
  CopRgt = ' 2007 - J. Rathlev, D-24147 Klausdorf';
  EMailAdr = 'J.Rathlev@schleswig-holstein.de';
  AppSubDir = 'JR-WeicheWare';    // Unterverz. in Anwendungsdaten fr Ini-Dateien
  }

type
  TDirection = (No,Up,Down,First,Last);

  TfmMain = class(TForm)
    btnInfo: TBitBtn;
    edtDir: THistoryCombo;
    btnDir: TSpeedButton;
    btnDirUp: TSpeedButton;
    btnQuit: TBitBtn;
    btnUpdate: TBitBtn;
    GroupBox2: TGroupBox;
    Panel1: TPanel;
    Panel2: TPanel;
    sbxGlyphs: TScrollBox;
    XPManifest1: TXPManifest;
    Label1: TLabel;
    btnGroup: TSpeedButton;
    btnDisable: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnDisableClick(Sender: TObject);
    procedure btnDirClick(Sender: TObject);
    procedure SpeedButtonClick(Sender: TObject);
    procedure cmCopyClick(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnDirUpClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure edtDirCloseUp(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnUpdateClick(Sender: TObject);
    procedure btnGroupClick(Sender: TObject);
  private
	{ Private-dclarations }
    IniName,sSearchPath,sSbName : String;
    FList                       : TStringList;
    SButtons                    : array of array of TSpeedButton;
    NCol,NRow,SPos              : integer;
    function LoadFileList : boolean;
    function ChangeDir : boolean;
  	procedure Draw;
  public
	{ Public-dclarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}

uses Clipbrd, WinDesktop, ShlObj, ShellDirDlg;

const
  (* INI-Sektionen *)
  CfGSekt = 'Config';

  iniPath   = 'Path';
  iniLeft   = 'Left';
  iniTop    = 'Top';
  iniHeight = 'Height';
  iniWidth  = 'Width';

procedure TfmMain.FormCreate(Sender: TObject);
var
  IniFile : TIniFile;
  AppPath : string;
  ok      : boolean;
begin
  FList:=TStringList.Create;
  AppPath:=GetDesktopFolder(CSIDL_APPDATA);
  if length(AppPath)>0 then begin
    AppPath:=SetDirName(AppPath)+AppSubDir;  // Pfad zu Anwendungsdaten
    ok:=ForceDirectories(AppPath);
    end
  else ok:=false;
  if not ok then AppPath:=GetDesktopFolder(CSIDL_PERSONAL);
  IniName:=Erweiter(AppPath,ExtractFileName(DelExt(Application.ExeName)),IniExt);
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    sSearchPath:=ReadString(CfgSekt,iniPath,'');
    Top:=ReadInteger (CfgSekt,iniTop,Top);
    Left:=ReadInteger (CfgSekt,iniLeft,Left);
    Height:=ReadInteger (CfgSekt,iniHeight,Height);
    Width:=ReadInteger (CfgSekt,iniWidth,Width);
    Free;
    end;
  edtDir.LoadFromIni (IniName,'History');
  edtDir.Text:=sSearchPath;
  NRow:=0; NCol:=0;
  SPos:=-1;
  end;

procedure TfmMain.FormShow(Sender: TObject);
begin
  ShellDirDialog.LoadFromIni(IniName,'Directories');
  if LoadFileList or ChangeDir then Draw;
  end;

procedure TfmMain.FormResize(Sender: TObject);
begin
  Draw;
  end;

procedure TfmMain.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  IniFile:=TIniFile.Create(IniName);
  with IniFile do begin
    WriteString (CfgSekt,iniPath,sSearchPath);
    WriteInteger (CfgSekt,iniLeft,Left);
    WriteInteger (CfgSekt,iniTop,Top);
    WriteInteger (CfgSekt,iniHeight,Height);
    WriteInteger (CfgSekt,iniWidth,Width);
    Free;
    end;
end;

(* load file list *)
function TfmMain.LoadFileList : boolean;
var
  tsRes : TSearchRec;
  rc    : integer;
begin
  rc:=FindFirst(sSearchPath+'*.bmp',faAnyFile,TsRes);
  if rc<>0 then Result:=false
  else with FList do begin
    Clear;
    while (rc=0) do begin
      Add(TsRes.Name);
      rc:=FindNext(TsRes);
      end;
    Sort;  
    Result:=true;
    end;
  end;

{ fill the speedbuttons with the glyphs from FList[n] }
procedure TfmMain.Draw;
var
	n,i,j :integer;
begin
  for j:=0 to NRow-1 do for i:=0 to NCol-1 do begin
    if assigned(SButtons[i,j]) then FreeAndNil(SButtons[i,j]);
    end;
  with sbxGlyphs do begin
    NCol:=(Width-BtnLeft-15) div (BtnWidth+BtnDist);
//    NRow:=(Height-BtnTop) div (BtnHeight+BtnDist);
//    if NCol*NRow<FList.Count then dec(NCol);  // mit Scrollbar
    NRow:=(FList.Count-1) div NCol +1;
    end;
  SetLength(SButtons,NCol,NRow);
  for j:=0 to NRow-1 do for i:=0 to NCol-1 do begin
    n:=j*NCol+i;
    if n<FList.Count then begin
      SButtons[i,j]:=TSpeedButton.Create(sbxGlyphs);
      with SButtons[i,j] do begin
        Parent:=sbxGlyphs;
        Width:=BtnWidth; Height:=BtnHeight;
        Left:=BtnLeft+i*(BtnWidth+BtnDist);
        Top:=BtnTop+j*(BtnHeight+BtnDist);
        AllowAllUp:=true;
        ShowHint:=true;
        OnClick:=SpeedButtonClick;
        with Glyph do begin
          LoadFromFile(sSearchPath+FList.Strings[n]);
          NumGlyphs:=Width div Height;
          end;
        Hint:=FList.Strings[n]+' ('+IntToStr(NumGlyphs)+')';
        Tag:=JoinWord(i,j);
        end;
      end;
    end;
  end;

procedure TfmMain.btnGroupClick(Sender: TObject);
var
	i : integer;
begin
	with btnGroup do if Down then begin
    Hint:='Set GroupIndex of Buttons = 0';
		end
  else begin
    Hint:='Set GroupIndex of Buttons = 1';
		end;
  if SPos>=0 then begin
    SButtons[HiWord(SPos),LoWord(SPos)].Down:=false;
    SPos:=-1;
    end;
	with sbxGlyphs do for i:=0 to ControlCount-1 do begin
		if Controls[i] is TSpeedButton then begin
			if btnGroup.Down then TSpeedButton(Controls[i]).GroupIndex:=1
      else with TSpeedButton(Controls[i]) do begin
        GroupIndex:=0; Down:=false;
        end;
      end;
  	end;
  if btnGroup.Down and (FList.Count>0) then begin
    SButtons[0,0].Down:=true;
    SPos:=0;
    end
  end;

procedure TfmMain.btnDisableClick(Sender: TObject);
var
	i : integer;
begin
	with btnDisable do if Down then begin
    Hint:='Enable glyphs';
		end
  else begin
    Hint:='Disable glyphs';
		end;
	with sbxGlyphs do for i:=0 to ControlCount-1 do begin
		if Controls[i] is TSpeedButton then
			TSpeedButton(Controls[i]).Enabled:=not btnDisable.Down;
  	end;
  end;

function TfmMain.ChangeDir : boolean;
begin
  if ShellDirDialog.Execute ('Select Directory',false,'',sSearchPath) then begin
    sSearchPath:=IncludeTrailingPathDelimiter(sSearchPath);
    with edtDir do begin
      Text:=sSearchPath; AddItem(sSearchPath);
      end;
    Result:=LoadFileList;
    end
  else Result:=false;
  end;

procedure TfmMain.btnDirClick(Sender: TObject);
begin
  if ChangeDir then Draw;
  end;

procedure TfmMain.SpeedButtonClick(Sender: TObject);
var
  s : string;
begin
  s:=(Sender as TSpeedButton).Hint;
  s:=Trim(ReadNxtStr(s,'('));
	ClipBoard.AsText:=sSearchPath+s;
  if btnGroup.Down then SPos:=(Sender as TSpeedButton).Tag;
  end;

procedure TfmMain.cmCopyClick(Sender: TObject);
begin
	ClipBoard.AsText:=sSearchPath+TSpeedButton(FindComponent(sSBName)).Hint;
  end;

procedure TfmMain.btnInfoClick(Sender: TObject);
begin
	InfoDialog('',Prog+Vers+#13+CopRgt+#13+'E-Mail: '+EMailAdr);
  end;

procedure TfmMain.btnDirUpClick(Sender: TObject);
var
  s : string;
begin
  s:=sSearchPath;
  sSearchPath:=ExtractFilePath(ExcludeTrailingPathDelimiter(sSearchPath));
  if LoadFileList or ChangeDir then begin
    Draw;
    with edtDir do begin
      Text:=sSearchPath; AddItem(sSearchPath);
      end;
    end
  else sSearchPath:=s;
  end;

procedure TfmMain.btnQuitClick(Sender: TObject);
begin
  Close;
  end;

procedure TfmMain.edtDirCloseUp(Sender: TObject);
var
  s : string;
begin
  with edtDir do if Items.Count>0 then begin
    s:=Text;
    sSearchPath:=Items[ItemIndex];
    if LoadFileList or ChangeDir then Draw
    else Text:=s;
    end;
  end;

procedure TfmMain.btnUpdateClick(Sender: TObject);
begin
  if LoadFileList then Draw;
  end;

end.
